﻿using System.Collections;
using System.Collections.Generic;
using System.Linq;
using System.Windows;
using Telerik.Windows.DragDrop.Behaviors;

namespace Telerik.Windows.Controls.GridView
{
	/// <summary>
	/// 
	/// </summary>
	public class GridViewDragDropHelper : DragDropHelper<GridViewDragDropBehavior, Telerik.Windows.DragDrop.Behaviors.DragDropState>
	{
		/// <summary>
		/// Returns a list containing the items of the specified RadGridView.
		/// </summary>
		protected override IList GetItemsSource(FrameworkElement control)
		{
			var gridView = control as RadGridView;

			return gridView == null ? null : (IList)gridView.ItemsSource;
		}

		protected override DependencyObject GetContainerFromItem(FrameworkElement control, object item)
		{
			return ((RadGridView)control).ItemContainerGenerator.ContainerFromItem(item);
		}

		/// <summary>
		/// Returns the SelectedItems list of the RadGridView that contains the dragged row and adds the dragged row to the list if it is not already present there.
		/// </summary>
		/// <param name="element">The source control of the drag operation.</param>
		/// <returns>The items that should be dragged.</returns>
		protected override IEnumerable<object> GetDraggedItems(FrameworkElement element)
		{
			var gridView = element.ParentOfType<RadGridView>();
			if (gridView != null)
			{
				var selectedItems = gridView.SelectedItems;
				var item = element.DataContext;

				if (!selectedItems.Contains(item))
				{
					if (gridView.SelectionMode == System.Windows.Controls.SelectionMode.Single)
					{
						gridView.SelectedItem = item;
					}
					else
					{
						selectedItems.Add(item);
					}
				}

				return selectedItems.OfType<object>().ToList();
			}
			return Enumerable.Empty<object>();
		}

		/// <summary>
		/// Finds the RadGridView that contains the provided element.
		/// </summary>
		/// <param name="element">The element that raised the DragInitialize event.</param>
		protected override FrameworkElement FindDragSource(FrameworkElement element)
		{
			return element as RadGridView ?? element.ParentOfType<RadGridView>();
		}

		/// <summary>
		/// Finds the GridViewRow that contains the provided element.
		/// </summary>
		/// <param name="element">The element that raised the DragInitialize event.</param>
		protected override FrameworkElement FindDraggedItem(FrameworkElement element)
		{
			return element as GridViewRow ?? element.ParentOfType<GridViewRow>();
		}

		/// <summary>
		/// Finds the RadGridView that should accept the dragged items.
		/// </summary>
		/// <param name="element">The element that raised the Drop event.</param>
		/// <param name="data">The DataObject for the current drag operation.</param>
		protected override FrameworkElement FindDropItemsControl(FrameworkElement element, object data)
		{
			return element as RadGridView ?? element.ParentOfType<RadGridView>();
		}

		/// <summary>
		/// Finds a RadGridView from the target control that contains the provided element.
		/// </summary>
		/// <param name="element">The element that raised the Drop event.</param>
		protected override FrameworkElement FindDropTarget(FrameworkElement element)
		{
			return element as RadGridView ?? element.ParentOfType<RadGridView>();
		}
	}
}