﻿using System.Linq;
using Telerik.Windows.DragDrop.Behaviors;

namespace Telerik.Windows.Controls.TreeView
{
	/// <summary>
	/// 
	/// </summary>
	public class TreeViewDragDropBehavior : DragDropBehavior<Telerik.Windows.DragDrop.Behaviors.DragDropState>
	{
		/// <summary>
		/// Initializes a new instance of the <see cref="TreeViewDragDropBehavior"/> class.
		/// </summary>
		public TreeViewDragDropBehavior()
		{
		}

		/// <summary>
		/// Completes the drop operation. This method is called only in the context of the drop target control.
		/// </summary>
		/// <param name="state">DragDropState that provides context for the current operation.</param>
		/// <remarks>
		/// When the drag source and the drop target are the same control, this method is called before DragCompleted.
		/// </remarks>
		public override void Drop(Telerik.Windows.DragDrop.Behaviors.DragDropState state)
		{
			var items = state.DraggedItems;
			var insertIndex = this.AllowReorder ? state.InsertIndex : -1;

			if (state.IsSameControl)
			{
				if (this.IsMovingItems(state))
				{
					if (insertIndex != -1)
					{
						insertIndex -= state.DraggedItems.OfType<object>().Count(item => state.DestinationItemsSource.IndexOf(item) < insertIndex);
					}

					RemoveItems(state.DestinationItemsSource, state.DraggedItems);
				}
				else
				{
					items = this.CopyDraggedItems(state);
				}
			}

			InsertItems(state.DestinationItemsSource, insertIndex, items);
		}
	}
}